format compact;
clc;
clear all

disp('More Topics about Matlab');
disp('==========================================');
disp('  1. Running Matlab');
disp('  2. More data structures and built-in functions');
disp('  3. More about M files');
disp('  4. Debugging your code');
disp('  5. Accelerating your code');
disp('  6. Plotting graphs');
disp('');
section=input('Select a Section [1-6]: ');
clc
    
if(section <=1);
    
disp('1. Running Matlab');
disp('---------------------');
input('');
disp(' * Running Matlab in two different mode:');
disp('       a. Default mode(with Java Virtual Machine): Matlab Command Window and Editor.');
disp('          -- providing some useful tools.');
disp('          -- using more memory and easier to break down in Unix.');
input('');
disp('       b. Without JVM: ''matlab -nojvm'' or ''matlab -nodesktop''.')
disp('          -- no matlab editor (Using your own editor).')
disp('          -- faster and more stable.')

input('');
clc
end

if(section<=2);

disp('2. More data structures and built-in functions');
disp('----------------------------------------------');
input('');
disp(' ***Define a Structure:***');
disp('---------------------------');
input('>> net.w = [0.1 0.2 0.3 0.4]');
net.w = [0.1 0.2 0.3 0.4;0.2 0.3 0.5 0.6]
input('>> net.b = zeros(1,4)');
net.b = zeros(1,4)
input('');

disp(' ***Define a Structure array:***');
disp('--------------------------------');
disp('>> for i = 1:10')
disp('      net2(i).w = ones(1,10);');
disp('   end');
for i = 1:10
    net2(i).w = ones(1,10);
end
input('>> net2');
net2
input('>> net2(1)');
net2(1)

input('');
disp(' ***Define a Cell:***');
disp('----------------------');
input('>> myCell = cell(1,3)')
myCell = cell(1,3)
disp('>> myCell{1} = ''This is a string''');
disp('>> myCell{2} = [1 2 3]');
input('>> myCell{3} = sqrt(-1)');
myCell{1} = 'This is a string';
myCell{2} = [1 2 3];
myCell{3} = sqrt(-1);
myCell
input('');

% some useful functions
disp(' ***Some useful built-in functions: ***');
disp('----------------------------------------');

disp('Get size of variables');
disp('  - length of vector: length()');
disp('  - size of matrix: size()');
disp('>> x = [1 2 3 4 5];');
input('>> length(x)');
x = [1 2 3 4 5];
length(x)

disp('Replicate and tile a matrix');
input('>> y = [1 2 -3]''');
y = [1 2 -3]'
input('>> repmat(y,1,2)')
repmat(y,1,2)

disp('Reshape a matrix');
disp('   - matrix to vector')
disp('');
input('>> x = [1 2 3 4; 2 4 6 8; -3 -6 -9 -12]');
x = [1 2 3 4; 2 4 6 8; -3 -6 -9 -12]
input('>> y = x(:)');
y = x(:)
disp('   - reshape matrix')
disp('');
input('>> reshape(y,3,4)')
reshape(y,3,4)

disp('Generate random numbers:')
disp('    - uniformly distributed: rand()');
input('>> x = rand(1,100);');
x = rand(1,500);
input('>> hist(x)');
hist(x);
input('');

disp('    - normally distributed: randn()');
input('>> y = randn(1,500);');
y = randn(1,500);
input('>> hist(y)');
hist(y)
input('');
input('>> hist(y,20)');
hist(y,20)
input('');

disp(' *Notice that each function usually can be used in several') 
disp('  different ways by giving different type of arguments. Use') 
disp('  ''help <functionName>'' to check the details') 
input('')

disp('Other functions: ')
disp('      eig - calculate eigenvalues and eigenvectors of a matrix');
disp('      diag - get diagonal elements of a matrix');
disp('      det - comput determinant of a square matrix');
input('');
clc
end

if(section <=3);
clc
disp('3. More about M files');
disp('----------------------');
input('');
% scripts
disp('*Script file:')
disp('   - No arguments')
disp('   - Command name = script filename')
disp('   - Variables are stored in the base workspace')
disp('   - Can call other M-files')
input('')

% functions
disp('*Function file:')
disp('   - Function is called according to its filename');
input('    and the first function in that file is called');
disp('   - ''nargin'': number of input arguments')
input('')
input('>> myrandint(2,2)')
myrandint(2,2)
input('>> myrandint(2,2,1,20)')
myrandint(2,2,1,20)
input('')
% local and global variable
disp('   - Each matlab function has its own local variables')
disp('   - Define a global variable using ''global <variable names>'' in your functions');
disp('   - Each global variable is visible to all the functions declaring it');
input('')

% functions as parameters
input('   - functions can have several outputs')
input('   - functions can be run in another way')
disp('>> magic(3)')
magic(3)
disp('>> eval('' magic(3)'')')
eval('magic(3)')
input('* Another example: See .m file')
disp('>> [mean1, var1] = funcdemo(''rand'',10)')
[mean1, var1] = funcdemo('rand',10)
input('')
disp('>> [mean2, var2] = funcdemo(''randn'',10)')
[mean2, var2] = funcdemo('randn',10)
input('')
clc
end

if(section <=4);
disp('4. Debugging your code')
disp('-------------------------------')
% command line
disp('Two different ways:')
disp('  - command line: using command ''debug'', ''dbstop'',''dbclear'',''dbstep''...')
disp('                  check help to see how to use them when no java machine is loaded')
disp('  - use matlab command window and editor altogether')
input('');
clc
return
end

if(section <=5);
disp('5. Accelerating your code');
disp('-----------------------------------');
disp('* Command for measuring the running time')
disp('  tic; <your code>; toc');
input('');
disp('* Some tips')
disp('1. Solve linear equations: A*x=b')
input('     use x=A\\b instead of x = inv(A)*b');
disp('2. Generate variables before loops');
input('>> tic; for i = 1:1000, newarray(i)= exp(sin(i)); end; toc')
tic; for i = 1:1000, newarray(i)= exp(sin(i)); end; toc
disp('>> tic; newarray = zeros(1,1000);...')
input('   for i = 1:1000, newarray(i)= exp(sin(i)); end; toc')
tic; newarray = zeros(1,1000); for i = 1:1000, newarray(i)= exp(sin(i)); end; toc
input('')
disp('* Vectorization')
disp('  - Transform your computation into matrix calculation as much as possible');
disp('  - Avoid using loops')
input('* Some examples');
disp('  1. Calculate distance between y0 and a set of points x');
disp('>> x = 0:0.1:1');
x = 0:0.1:1
disp('>> y0 = 5');
y0 = 5
input('')
disp('Using loop is easy, but it can be vectorized:')
input('>> dist = abs(x - repmat(y0,1,length(x)))');
dist = abs(x - repmat(y0,1,length(x)))
input('')
disp('  2. Change matrix elements')
disp('>> x = magic(8)')
x = magic(8)
input(' replace any element larger than 25 with -1');
disp('>> index = find(x>25);');
disp('>> x(index) = -1');
index = find(x>25);
x(index) = -1
input('')
clc
end

if(section <=6);
disp('6. Plotting Graphs');
disp('------------------');
input('');
disp('* Generate several figures at the same time:');
disp('  - in different windows: figure(#win)');
input('>> figure(1)')
figure(1);
disp('>> x = 0:0.01:2*pi');
x = 0:0.01:2*pi;
disp('>> plot(x,sin(x))');
plot(x,sin(x));
input('>> figure(2)');
figure(2)
disp('>> plot(x,cos(x))');
plot(x,cos(x));
input('');
close all
disp('  - in the same window: subplot');
input('>> subplot(1,2,1)');
subplot(1,2,1);
disp('>> plot(x,sin(x))');
plot(x,sin(x));
input('>> subplot(1,2,2)');
subplot(1,2,2);
disp('>> plot(x,cos(x))');
plot(x,cos(x));
input('');
close all
disp('* Place strings into graph')
disp('>> plot(x,sin(x))');
plot(x,sin(x));
input('>> gtext(''You are here'')');
gtext('You are here');
input('')

close all
input('* Draw points');
disp('>> x = randn(1,200)');
x = randn(1,200);
disp('>> y = randn(1,200)');
y = randn(1,200);
disp('scatter(x,y)');
scatter(x,y)
input('');
hold on
disp('>> x1 = randn(1,200)+5');
x1 = randn(1,200)+5;
disp('>> y1 = randn(1,200)+4');
y1 = randn(1,200)+4;
disp('scatter(x1,y1,''r+'')');
scatter(x1,y1,'r+')
hold off
disp('');

input('* Control axis');
input('>> axis off');
axis off;
input('>> axis on');
axis on
input('>> axis equal');
axis equal
input('');

close all
disp('* 3D graph');
disp(' - plot3(x,y,z)');
disp('>> t = 0.01:0.01:20*pi;');
input('>> plot3(cos(t),sin(t),t.^3)');
t = 0.01:0.01:20*pi;
plot3(cos(t),sin(t),t.^3)
input('');
disp(' - surface plot');
disp('>> xx = -2:.2:2;');
disp('>> yy = xx;');
disp('>> [x,y] = meshgrid(xx,yy);');
disp('>> z = exp(-x.^2-y.^2);');
input('>> mesh(z);');
xx = -2:.2:2;
yy = xx;
[x,y] = meshgrid(xx,yy);
z = exp(-x.^2-y.^2);
mesh(z);
input('>> surf(z)');
surf(z);
disp('* You can change the ''colormap'', ''shading'' and ''view''')
input('');
clc
end